
public class Optimierung
{
    //RungeKuttaIntegrator ruku = new RungeKuttaIntegrator(new SchwingerModell());
    RungeKuttaIntegrator ruku;
    AutonomesVehikel modell;
    double[] yalt,yneu;
    double x,y,phi,vx,vy,omega,t,dt; //Aktueller Zustand des Fahrzeugs, Meter Meter rad
    
    public Optimierung()
    {
        Par.initialisieren(640,480);
        
        modell = new AutonomesVehikel();
        ruku = new RungeKuttaIntegrator(modell);

        yalt = new double[modell.holeAnzahlGleichungen()];
        yneu = new double[modell.holeAnzahlGleichungen()];
        initialisieren();
        
    }
    
    public void initialisieren()
    {
            x=0.0;
            y=-0.3;
            phi=0.0;
            vx=0.0;
            vy=0.0;                        
            omega=0.0;
            t=0.0;
            dt=0.01;
        
            yalt[0]=x;
            yalt[1]=y;
            yalt[2]=phi;
            yalt[3]=vx;
            yalt[4]=vy;
            yalt[5]=omega;
            
            for(int i=0;i<yalt.length;i++)
                yneu[i]=yalt[i];
    }

    public double berechneFehler(double[] par)
    {
        boolean FERTIG = false;
        initialisieren();
        modell.PWERT    = par[0];
        modell.SOLLWERT = par[1];
        
        while(!FERTIG)
        {
            for(int i=0;i<10;i++)
            {
                for(int k=0;k<yalt.length;k++)
                   yalt[k]=yneu[k];
                yneu = ruku.berechneZeitschritt(yalt,t,dt);    
                t+=dt;            
            }
            x=yalt[0];
            y=yalt[1];
            phi=yalt[2];
            vx=yalt[3];
            vy=yalt[4];
            omega=yalt[5];        
            
            modell.aktualisiereSensorwert(x,y,phi);
            
            if(modell.rundeVollendet(x,y,phi))
            {
                FERTIG=true;
            }
            
            if(modell.kollision(x,y,phi))    
            {
                //FERTIG=true;
                t+=100.0*dt; //>> als gewöhnliche Rundenzeit als "Strafe"               
            }
            
            
        } 
        
        return t;
    }
    
    public static int findeKleinstesIndex(double[] werte)
    {
        for(int i=0;i<werte.length;i++)
        {
            boolean gefunden = true;
            for(int k=0;k<werte.length;k++)
            {
                if(i!=k && werte[k]<werte[i])
                    gefunden=false;
            }
            if(gefunden==true)
            {
                return i;
            }
        }
        return 0;
    }
    
    public static void main(String[] args)
    {
//        double PWERT = 0.0005;
        double PWERT = 0.00055;
        double SOLLWERT = 50.0;
        
        double[] schrittweite = {PWERT*0.01,SOLLWERT*0.01};
        
        Optimierung opt = new Optimierung();
        
        double[] par = {PWERT,SOLLWERT};
        double[][] par_neu = new double[9][2];
        double[] fehler_aktuell = new double[9];
        
        double testfehler = opt.berechneFehler(par);
        System.out.println("FEHLER = "+testfehler);

        for(int durchlauf = 0; durchlauf<200;durchlauf++)
        {
            int p=0;
            for(int i=-1;i<=1;i++)
            {
                for(int k=-1;k<=1;k++)
                {
                    par_neu[p][0]=par[0]+(double)i*schrittweite[0];
                    par_neu[p][1]=par[1]+(double)k*schrittweite[1];
                    fehler_aktuell[p] = opt.berechneFehler(par_neu[p]);
                    p++;
                }
            }
            int best_index = findeKleinstesIndex(fehler_aktuell);
System.out.println("Durchlauf "+durchlauf+ " best_index="+best_index);
//            if(best_index==0)
            if(fehler_aktuell[best_index]>=testfehler)
                best_index=4;
            if(best_index==4)
            {
                for(int i=0;i<schrittweite.length;i++)
                    schrittweite[i]*=0.9;
                              
                par[0] = par_neu[best_index][0];
                par[1] = par_neu[best_index][1];
                System.out.println("PWERT="+par[0]+" SOLLW="+par[1]+" FEHLER = "+testfehler);
                testfehler = opt.berechneFehler(par);
            }
            else
            {
                par[0] = par_neu[best_index][0];
                par[1] = par_neu[best_index][1];
                testfehler = opt.berechneFehler(par);
                System.out.println("PWERT="+par[0]+" SOLLW="+par[1]+" FEHLER = "+testfehler);
                for(int i=0;i<schrittweite.length;i++)
                    schrittweite[i]*=1.1;
            }
        }
    }
}

